@@ -9,7 +9,15 @@ from typing import Dict, Iterator, List, Tuple, Union
 from pynmrstar import Entry, Loop, Saveframe
 from strenum import LowercaseStrEnum
 
-from nef_pipelines.lib.util import exit_error, is_float, is_int, running_in_pycharm
+from nef_pipelines.lib.constants import NEF_PIPELINES, NEF_PIPELINES_VERSION
+from nef_pipelines.lib.util import (
+    exit_error,
+    fixup_metadata,
+    is_float,
+    is_int,
+    running_in_pycharm,
+    script_name,
+)
 
 NEF_CATEGORY_ATTR = "__NEF_CATEGORY__"
 NEF_MOLECULAR_SYSTEM = "nef_molecular_system"
@@ -18,6 +26,16 @@ NEF_METADATA = "nef_nmr_meta_data"
 UNUSED = "."
 
 
+class BadNefFileException(Exception):
+    """
+    marker class for star files that don't conform to NEF conventions, e.g. files with multiple singletons
+    as an example the molecular system should be a singleton
+    ths doesn't imply the file has invalid star syntax, the star syntax will be valid
+    """
+
+    pass
+
+
 # what the selection type is for Star SaveFrames
 class SelectionType(LowercaseStrEnum):
     NAME = auto()
@@ -190,7 +208,11 @@ def loop_row_dict_iter(
     """
 
     if not isinstance(loop, Loop):
-        raise Exception(f"loop must be loop you presented {type(loop)}")
+        msg = f"""\
+            loop must be of type Loop you provided a {loop.__class__.__name__}"
+            value: {loop}
+        """
+        raise Exception(msg)
 
     for row in loop:
         row = {tag: value for tag, value in zip(loop.tags, row)}
@@ -307,3 +329,96 @@ def file_name_path_to_frame_name(path: str) -> str:
     stem = stem.replace(".", "_")
 
     return stem
+
+
+def molecular_system_from_entry_or_exit(entry: Entry):
+    """
+    read the nef molecular system [a singleton] from a star entry or exit with a helpful message
+    :param entry: The NEF star entry
+    :return: the molecular system save frame or None if there isn't one
+    """
+
+    molecular_system = None
+    try:
+        molecular_system = molecular_system_from_entry(entry)
+    except BadNefFileException as e:
+        exit_error(str(e))
+
+    if not molecular_system:
+        exit_error(
+            "Couldn't find a molecular system frame it should be labelled 'save_nef_molecular_system'"
+        )
+
+    return molecular_system
+
+
+def molecular_system_from_entry(entry):
+    """
+    read the nef molecular system [a singleton] from a star entry or raise a BadNefFileException
+    :param entry: The NEF star entry
+    :return: the molecular system save frame or None if there isn't one
+    """
+    result = None
+
+    molecular_systems = entry.get_saveframes_by_category(NEF_MOLECULAR_SYSTEM)
+    if len(molecular_systems) == 1:
+        result = molecular_systems[0]
+    elif len(molecular_systems) > 1:
+        msg = f"""\
+                there must be only one molecular_system i found {len(molecular_systems)}
+                frame_names are {', '.join([frame.name for frame in molecular_systems])}
+        """
+        raise BadNefFileException(msg)
+
+    return result
+
+
+def add_frames_to_entry(entry: Entry, frames: List[Saveframe]) -> Entry:
+    # TODO deal with merging esp wrt to molecular systems and possibly with other information
+    # TODO add frame rename and frame delete
+    """
+    take a set of save frames and  add them to an Entry and update the NEF metadata header
+
+    Args:
+        entry: an entry to add the save frames to
+        frames: a set of save frames to add, they must have different names to those present already
+
+
+    Returns:
+        the updated entry containing the frames
+    """
+
+    fixup_metadata(entry, NEF_PIPELINES, NEF_PIPELINES_VERSION, script_name(__file__))
+
+    for frame in frames:
+
+        new_frame_name = frame.name
+
+        frame_in_entry = is_save_frame_name_in_entry(entry, new_frame_name)
+
+        if frame_in_entry:
+            msg = (
+                f"the frame named {new_frame_name} already exists in the stream, rename it or delete to add "
+                f"the new frame shift frame"
+            )
+            exit_error(msg)
+
+        entry.add_saveframe(frame)
+
+    return entry
+
+
+def is_save_frame_name_in_entry(entry: Entry, frame_name: str) -> bool:
+    """
+    check if a save frame name exists in an entry
+    :param entry: the entry
+    :param frame_name: the frame name
+    :return: if a savefame with the name is present
+    """
+    frame_in_entry = False
+    try:
+        entry.get_saveframe_by_name(frame_name)
+    except KeyError:
+        frame_in_entry = False
+
+    return frame_in_entry
