@@ -76,12 +76,14 @@ def main():
         import nef_pipelines.tools.header  # noqa: F401
         import nef_pipelines.tools.stream  # noqa: F401
         import nef_pipelines.tools.test  # noqa: F401
+        import nef_pipelines.transcoders.csv  # noqa: F401
         import nef_pipelines.transcoders.fasta  # noqa: F401
         import nef_pipelines.transcoders.mars  # noqa: F401
         import nef_pipelines.transcoders.nmrpipe  # noqa: F401
         import nef_pipelines.transcoders.nmrview  # noqa: F401
         import nef_pipelines.transcoders.pales  # noqa: F401
         import nef_pipelines.transcoders.pdbx  # noqa: F401
+        import nef_pipelines.transcoders.sparky  # noqa: F401
         import nef_pipelines.transcoders.xplor  # noqa: F401
 
     except Exception as e:
